// Filename: pystub.cxx
// Created by:  drose (09Aug00)
//
////////////////////////////////////////////////////////////////////
//
// PANDA 3D SOFTWARE
// Copyright (c) 2001 - 2004, Disney Enterprises, Inc.  All rights reserved
//
// All use of this software is subject to the terms of the Panda 3d
// Software license.  You should have received a copy of this license
// along with this source code; you will also find a current copy of
// the license at http://etc.cmu.edu/panda3d/docs/license/ .
//
// To contact the maintainers of this program write to
// panda3d-general@lists.sourceforge.net .
//
////////////////////////////////////////////////////////////////////

#include "pystub.h"

extern "C" {
  EXPCL_DTOOLCONFIG int PyArg_ParseTuple(...);
  EXPCL_DTOOLCONFIG int PyErr_Occurred(...);
  EXPCL_DTOOLCONFIG int PyErr_SetString(...);
  EXPCL_DTOOLCONFIG int PyExc_TypeError(...);
  EXPCL_DTOOLCONFIG int PyDict_GetItem(...);
  EXPCL_DTOOLCONFIG int PyFloat_AsDouble(...);
  EXPCL_DTOOLCONFIG int PyFloat_FromDouble(...);
  EXPCL_DTOOLCONFIG int PyFloat_Type(...);
  EXPCL_DTOOLCONFIG int PyInt_AsLong(...);
  EXPCL_DTOOLCONFIG int PyInt_FromLong(...);
  EXPCL_DTOOLCONFIG int PyInt_Type(...);
  EXPCL_DTOOLCONFIG int PyList_Append(...);
  EXPCL_DTOOLCONFIG int PyList_AsTuple(...);
  EXPCL_DTOOLCONFIG int PyList_New(...);
  EXPCL_DTOOLCONFIG int PyLong_AsLongLong(...);
  EXPCL_DTOOLCONFIG int PyLong_AsUnsignedLongLong(...);
  EXPCL_DTOOLCONFIG int PyLong_FromLongLong(...);
  EXPCL_DTOOLCONFIG int PyLong_FromUnsignedLong(...);
  EXPCL_DTOOLCONFIG int PyLong_FromUnsignedLongLong(...);
  EXPCL_DTOOLCONFIG int PyLong_Type(...);
  EXPCL_DTOOLCONFIG int PyNumber_Long(...);
  EXPCL_DTOOLCONFIG int PyObject_CallObject(...);
  EXPCL_DTOOLCONFIG int PyObject_GetAttrString(...);
  EXPCL_DTOOLCONFIG int PyObject_HasAttrString(...);
  EXPCL_DTOOLCONFIG int PyObject_IsInstance(...);
  EXPCL_DTOOLCONFIG int PyObject_IsTrue(...);
  EXPCL_DTOOLCONFIG int PyObject_SetAttrString(...);
  EXPCL_DTOOLCONFIG int PyObject_Str(...);
  EXPCL_DTOOLCONFIG int PySequence_Check(...);
  EXPCL_DTOOLCONFIG int PySequence_GetItem(...);
  EXPCL_DTOOLCONFIG int PySequence_Size(...);
  EXPCL_DTOOLCONFIG int PySequence_Tuple(...);
  EXPCL_DTOOLCONFIG int PyString_AsString(...);
  EXPCL_DTOOLCONFIG int PyString_AsStringAndSize(...);
  EXPCL_DTOOLCONFIG int PyString_FromString(...);
  EXPCL_DTOOLCONFIG int PyString_FromStringAndSize(...);
  EXPCL_DTOOLCONFIG int PyString_Type(...);
  EXPCL_DTOOLCONFIG int PyTuple_GetItem(...);
  EXPCL_DTOOLCONFIG int PyTuple_New(...);
  EXPCL_DTOOLCONFIG int PyType_IsSubtype(...);
  EXPCL_DTOOLCONFIG int PyUnicode_Type(...);
  EXPCL_DTOOLCONFIG int Py_BuildValue(...);
  EXPCL_DTOOLCONFIG int Py_InitModule4(...);

  EXPCL_DTOOLCONFIG extern void *PyExc_AssertionError;
  EXPCL_DTOOLCONFIG extern void *_Py_NoneStruct;
};

int PyArg_ParseTuple(...) { return 0; }
int PyErr_Occurred(...) { return 0; }
int PyErr_SetString(...) { return 0; }
int PyExc_TypeError(...) { return 0; }
int PyDict_GetItem(...) { return 0; }
int PyFloat_AsDouble(...) { return 0; }
int PyFloat_FromDouble(...) { return 0; }
int PyFloat_Type(...) { return 0; }
int PyInt_AsLong(...) { return 0; }
int PyInt_FromLong(...) { return 0; }
int PyInt_Type(...) { return 0; }
int PyList_Append(...) { return 0; }
int PyList_AsTuple(...) { return 0; }
int PyList_New(...) { return 0; }
int PyLong_AsLongLong(...) { return 0; }
int PyLong_AsUnsignedLongLong(...) { return 0; }
int PyLong_FromLongLong(...) { return 0; }
int PyLong_FromUnsignedLong(...) { return 0; }
int PyLong_FromUnsignedLongLong(...) { return 0; }
int PyLong_Type(...) { return 0; }
int PyNumber_Long(...) { return 0; }
int PyObject_CallObject(...) { return 0; }
int PyObject_GetAttrString(...) { return 0; }
int PyObject_HasAttrString(...) { return 0; }
int PyObject_IsInstance(...) { return 0; }
int PyObject_IsTrue(...) { return 0; }
int PyObject_SetAttrString(...) { return 0; }
int PyObject_Str(...) { return 0; }
int PySequence_Check(...) { return 0; }
int PySequence_GetItem(...) { return 0; }
int PySequence_Size(...) { return 0; }
int PySequence_Tuple(...) { return 0; }
int PyString_AsString(...) { return 0; }
int PyString_AsStringAndSize(...) { return 0; }
int PyString_FromString(...) { return 0; }
int PyString_FromStringAndSize(...) { return 0; }
int PyString_Type(...) { return 0; }
int PyTuple_GetItem(...) { return 0; }
int PyTuple_New(...) { return 0; }
int PyType_IsSubtype(...) { return 0; }
int PyUnicode_Type(...) { return 0; }
int Py_BuildValue(...) { return 0; }
int Py_InitModule4(...) { return 0; }

void *PyExc_AssertionError = (void *)NULL;
void *_Py_NoneStruct = (void *)NULL;


void
pystub() {
}
